/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *  CPPSTACK.CPP  --  stack class definitions                *
 *							     *
 *  Language:      Borland C++                               *
 *  Platform:      Gateway 386/25, MS-DOS 5.0                *
 *  Application:   CSE 687                                   *
 *  Author:        Jim Fawcett, CST 2-127, (315) 443-4454    *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <iostreams.h>     // declare cout, cerr, operator<<()
#include <stdlib.h>        // declare exit(1)
#include "cppstack.hpp"    // declare stack class

//----< stack::stack(n) >------------------------------------//
//      create empty stack as a dynamic array of n integers

stack::stack(int n) {  // Stack size can be determined at run time,
		       // before first use, but can't be extended.
		       // Need linked list definition to extend.
  if (!(stk = new int[n])) {
    cerr << "dynamic memory exhausted\n";
    exit(1);
  }
  size = n;
  top = 0;
}

//----< stack::~stack(void) >--------------------------------//
//      return dynamic memory to free list

stack::~stack(void) { delete [] stk; }

//----< stack::push(int datum) >-----------------------------//
//      push integer datum on stack

void stack::push(int datum) {

  if (top > size-1)
    cerr << "attempt to push beyond stack limits - datum lost\n";
  else
    stk[top++] = datum;
}

//----< stack::pop(void) >-----------------------------------//
//      remove integer datum from top of stack

int stack::pop(void) {

  if (top <= 0) {
    cerr << "attempt to pop datum from empty stack - zero returned\n";
    return 0;
  }
  else return stk[--top];
}
/**/
//----< stack::show_stack() >--------------------------------//
//      put stack contents on stdout

void stack::show_stack() {

  #define step 20
  for (int i=0; i<top; i++) {
    cout << stk[i] << "  ";
    if (i!=0 && i%step)
      cout << "\n";
  }
}

//----< stack::scan(int datum) >-----------------------------//
//      report depth at which datum is found on stack
//      report -1 if not found

int stack::scan(int datum) {

  for (int i=0; i<top; i++)
    if (stk[i] == datum)
      return i;
  return -1;
}

/**/
//----< test stub >------------------------------------------//

#ifdef test_stack

#include <fstream.h>    // declare ifstream class
#include <string.h>     // declare strlen()

int main(int argc, char *argv[]) {

  if (argc < 2) {
    cerr << "please enter test filename on command line\n";
    return 1;
  }
  ifstream istrm(argv[1]);
  if (!istrm) {
    cerr << "can't open " << argv[1] << "\n";
    return 1;
  }

  #define MAXSIZE 80
  char line[MAXSIZE];          // buffer to capture input lines
  stack instack(MAXSIZE);      // stack used to reverse lines
  stack outstack(MAXSIZE);     // stack used to reverse again

  while (istrm.getline(line,MAXSIZE)) {
    int len = strlen(line);
    for (int i=0; i<len; i++)   // push input char's on stack
      instack.push(line[i]);
    char ch;
    for (i=0; i<len; i++) {     // send to output in reverse order
      cout << (ch = (char)instack.pop());
      outstack.push((int)ch);   // reverse again
    }
    cout << "\n";
    for (i=0; i<len; i++)       // send out reverse reversed
      cout << (char)outstack.pop();
    cout << "\n";
  }
  return 0;
}
#endif
